@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_client_c =
- *   "$Id: client.c,v 1.4 2002/01/15 07:32:59 mwedel Exp $";
+ *   "$Id: client.c,v 1.10 2002/05/31 03:44:54 mwedel Exp $";
  */
 /*
     Crossfire client, a client program for the crossfire program.
@@ -51,14 +51,12 @@
 char *server=SERVER,*client_libdir=NULL,*meta_server=META_SERVER;
 char *image_file="";
 
-int port_num=EPORT, meta_port=META_PORT, want_skill_exp=0, mapx=11, mapy=11,
-    want_mapx=11, want_mapy=11, want_darkness=1, fog_of_war=0;
+int meta_port=META_PORT, want_skill_exp=0,
+    replyinfo_status=0, requestinfo_sent=0, replyinfo_last_face=0,
+    maxfd,map1cmd=0,metaserver_on=METASERVER;
 FILE *fpin,*fpout;
-int fdin, fdout, basenrofpixmaps, pending_images=0,maxfiledescriptor,
-	pending_archs=0,maxfd,map1cmd=0,metaserver_on=METASERVER;
 Client_Player cpl;
 ClientSocket csocket;
-int fast_tcp_send=1;
 
 char *resists_name[NUM_RESISTS] = {
 "armor", "magic", "fire", "elec", 
@@ -83,14 +81,14 @@ struct CmdMapping commands[] = {
     /* Order of this table doesn't make a difference.  I tried to sort
      * of cluster the related stuff together.
      */
-    { "map", MapCmd },
     { "map1", Map1Cmd },
+    { "map1a", Map1aCmd },
     { "map_scroll", (CmdProc)map_scrollCmd },
     { "magicmap", MagicMapCmd},
     { "newmap", NewmapCmd },
 
-    { "item", ItemCmd },
     { "item1", Item1Cmd },
+    { "item2", Item2Cmd },
     { "upditem", UpdateItemCmd },
     { "delitem", DeleteItem },
     { "delinv",	DeleteInventory },
@@ -99,8 +97,10 @@ struct CmdMapping commands[] = {
     { "stats", StatsCmd },
 
     { "image", ImageCmd },
+    { "image2", Image2Cmd },
     { "face", FaceCmd},
     { "face1", Face1Cmd},
+    { "face2", Face2Cmd},
 
 
     { "sound", SoundCmd},
@@ -116,6 +116,7 @@ struct CmdMapping commands[] = {
     { "setup", (CmdProc)SetupCmd},
 
     { "query", (CmdProc)handle_query},
+    { "replyinfo", (CmdProc)ReplyInfoCmd},
 };
 
 #define NCOMMANDS (sizeof(commands)/sizeof(struct CmdMapping))
@@ -209,7 +210,7 @@ int init_connection(char *host, int port)
 
 #ifdef TCP_NODELAY
     /* turn off nagle algorithm */
-    if (fast_tcp_send) {
+    if (use_config[CONFIG_FASTTCP]) {
 	int i=1;
 
 	if (setsockopt(fd, SOL_TCP, TCP_NODELAY, &i, sizeof(i)) == -1)
@@ -235,8 +236,6 @@ int init_connection(char *host, int port)
 
 void negotiate_connection(int sound)
 {
-    int cache;
-
     SendVersion(csocket);
 
     /* We need to get the version command fairly early on because
@@ -250,44 +249,96 @@ void negotiate_connection(int sound)
 	DoClient(&csocket);
     }
 
-
-    cache = display_willcache();
-    if (cache) cache = CF_FACE_CACHE;
-
     if (csocket.sc_version<1023) {
 	fprintf(stderr,"Server does not support PNG images, yet that is all this client\n");
 	fprintf(stderr,"supports.  Either the server needs to be upgraded, or you need to\n");
 	fprintf(stderr,"downgrade your client.\n");
 	exit(1);
     }
-    /* Other than cache, this doesn't do anything.  However, in the
-     * future, it may be possible to select other image sets (eg, iso)
+
+    /* If the user has specified a numeric face id, use it. If it is a string
+     * like base, then that resolves to 0, so no real harm in that.
      */
-    SendSetFaceMode(csocket,CF_FACE_PNG | cache);
-
-#if 0
-    if (display_usebitmaps()) 
-	SendSetFaceMode(csocket,CF_FACE_BITMAP | cache); 
-    else if (display_usexpm()) 
-	SendSetFaceMode(csocket,CF_FACE_XPM | cache);
-    else if (display_usepng()) {
-	if (csocket.sc_version<1023) {
-	    SendSetFaceMode(csocket,CF_FACE_XPM | cache);
-	    draw_info("Server does not support PNG images.  Will use XPM instead", NDI_RED);
-	}
-	else SendSetFaceMode(csocket,CF_FACE_PNG | cache);
-    }
-#endif
-
+    if (face_info.want_faceset) face_info.faceset = atoi(face_info.want_faceset);
     cs_print_string(csocket.fd,
-		    "setup sound %d sexp %d darkness %d newmapcmd %d",
-		    sound>=0, want_skill_exp, want_darkness, fog_of_war);
+	    "setup map1acmd 1 sound %d sexp %d darkness %d newmapcmd 1 faceset %d facecache %d itemcmd 2",
+	    sound>=0, want_skill_exp, 
+		    want_config[CONFIG_FOGWAR], face_info.faceset,
+		    want_config[CONFIG_CACHE]);
 
-    mapx=11;
-    mapy=11;
-    if (want_mapx!=11 || want_mapy!=11)
-	cs_print_string(csocket.fd,"setup mapsize %dx%d",want_mapx, want_mapy);
+    use_config[CONFIG_MAPHEIGHT]=want_config[CONFIG_MAPHEIGHT];
+    use_config[CONFIG_MAPWIDTH]=want_config[CONFIG_MAPWIDTH];
+    if (use_config[CONFIG_MAPHEIGHT]!=11 || use_config[CONFIG_MAPWIDTH]!=11)
+	cs_print_string(csocket.fd,"setup mapsize %dx%d",use_config[CONFIG_MAPWIDTH], use_config[CONFIG_MAPHEIGHT]);
 
+
+    /* If the server will answer the requestinfo for image_info and image_data,
+     * send it and wait for the response.
+     */
+    if (csocket.sc_version >= 1027) {
+	/* last_start is -99.  This means the first face requested will
+	 * be 1 (not 0) - this is OK because 0 is defined as the blank
+	 * face.
+	 */
+	int last_end=0, last_start=-99;
+
+	cs_print_string(csocket.fd,"requestinfo image_info");
+	requestinfo_sent = RI_IMAGE_INFO;
+	replyinfo_status = 0;
+	replyinfo_last_face = 0;
+
+	do {
+	    DoClient(&csocket);
+	    if (use_config[CONFIG_DOWNLOAD]) {
+		/* we need to know how many faces to
+		 * be able to make the request intelligently.
+		 * So only do the following block if we have that info.
+		 * By setting the sent flag, we will never exit
+		 * this loop until that happens.
+		 */
+		requestinfo_sent |= RI_IMAGE_SUMS;
+		if (face_info.num_images != 0) {
+		    /* Sort of fake things out - if we have sent the
+		     * request for image sums but have not got them all answered
+		     * yet, we then clear the bit from the status
+		     * so we continue to loop.
+		     */
+		    if (last_end == face_info.num_images) {
+			/* Mark that we're all done */
+			if (replyinfo_last_face == last_end) {
+			    replyinfo_status |= RI_IMAGE_SUMS;
+			    image_update_download_status(face_info.num_images, face_info.num_images, face_info.num_images);
+			}
+		    } else {
+			/* If we are all caught up, request another
+			 * 100 sums.
+			 */
+			if (last_end == replyinfo_last_face) {
+			    last_start += 100;
+			    last_end += 100;
+			    if (last_end > face_info.num_images) last_end = face_info.num_images;
+			    cs_print_string(csocket.fd,"requestinfo image_sums %d %d", last_start, last_end);
+			    image_update_download_status(last_start, last_end, face_info.num_images);
+			}
+		    }
+		} /* Still have image_sums request to send */
+	    } /* endif download all faces */
+	} while (replyinfo_status != requestinfo_sent);
+    }
+    if (use_config[CONFIG_DOWNLOAD]) {
+	char buf[MAX_BUF];
+
+	sprintf(buf,"Download of images complete.  Found %d locally, downloaded %d from server\n",
+		face_info.cache_hits, face_info.cache_misses);
+	draw_info(buf, NDI_GOLD);
+    }
+
+    /* This needs to get changed around - we really don't want to send
+     * the SendAddMe until we do all of our negotiation, which may include
+     * things like downloading all the images and whatnot - this is more an
+     * issue if the user is not using the default face set, as in that case,
+     * we might end up building images from the wrong set.
+     */
     SendAddMe(csocket);
 }
 
