@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_commands_c =
- *   "$Id: commands.c,v 1.3 2002/01/15 07:32:59 mwedel Exp $";
+ *   "$Id: commands.c,v 1.7 2002/05/31 03:44:54 mwedel Exp $";
  */
 /*
     Crossfire client, a client program for the crossfire program.
@@ -69,6 +69,36 @@
 #include <external.h>
 
 
+/* handles the response from a 'requestinfo' command.  This function doesn't
+ * do much itself other than dispatch to other functions.
+ */
+
+void ReplyInfoCmd(char *buf, int len)
+{
+    char *cp;
+    int i;
+
+    for (i=0; i<len; i++) {
+	/* Either a space or newline represents a break */
+	if (*(buf+i) == ' ' || *(buf+i) == '\n' ) break;
+    }
+    if (i>=len) {
+	/* Don't print buf, as it may contain binary data */
+	fprintf(stderr,"ReplyInfoCmd: Never found a space in the replyinfo\n");
+	return;
+    }
+    /* Null out the space and put cp beyond it */
+    cp = buf + i;
+    *cp++ = '\0';
+    if (!strcmp(buf,"image_info")) {
+	get_image_info(cp, len - i - 1);   /* located in common/image.c */
+    }
+    else if (!strcmp(buf,"image_sums")) {
+	get_image_sums(cp, len - i - 1);   /* located in common/image.c */
+    }
+}
+    
+
 /* Received a response to a setup from the server.
  * This function is basically the same as the server side function - we
  * just do some different processing on the data.
@@ -113,7 +143,7 @@ void SetupCmd(char *buf, int len)
 	 */
 	if (!strcmp(cmd,"sound")) {
 	    if (!strcmp(param,"FALSE"))
-		cs_print_string(csocket.fd, "setsound %d", !nosound);
+		cs_print_string(csocket.fd, "setsound %d", want_config[CONFIG_SOUND]);
 	} else if (!strcmp(cmd,"mapsize")) {
 	    int x, y=0;
 	    char *cp,tmpbuf[MAX_BUF];
@@ -121,9 +151,9 @@ void SetupCmd(char *buf, int len)
 	    if (!strcasecmp(param, "false")) {
 		draw_info("Server only supports standard sized maps (11x11)", NDI_RED);
 		/* Do this because we may have been playing on a big server before */
-		mapx = 11;
-		mapy = 11;
-		resize_map_window(mapx,mapy);
+		use_config[CONFIG_MAPWIDTH]=11;
+		use_config[CONFIG_MAPHEIGHT]=11;
+		resize_map_window(use_config[CONFIG_MAPWIDTH], use_config[CONFIG_MAPHEIGHT]);
 		continue;
 	    }
 	    x = atoi(param);
@@ -136,19 +166,17 @@ void SetupCmd(char *buf, int len)
 	     * size to server maximum, and re-sent the setup command.
 	     * Update our want sizes, and also tell the player what we are doing
 	     */
-	    if (want_mapx > x || want_mapy > y) {
-		if (want_mapx > x) want_mapx = x;
-		if (want_mapy > y) want_mapy = y;
+	    if (use_config[CONFIG_MAPWIDTH] > x || use_config[CONFIG_MAPHEIGHT] > y) {
+		if (use_config[CONFIG_MAPWIDTH] > x) use_config[CONFIG_MAPWIDTH] = x;
+		if (use_config[CONFIG_MAPHEIGHT] > y) use_config[CONFIG_MAPHEIGHT] = y;
 		cs_print_string(csocket.fd,
-				"setup mapsize %dx%d", want_mapx, want_mapy);
+				"setup mapsize %dx%d", use_config[CONFIG_MAPWIDTH], use_config[CONFIG_MAPHEIGHT]);
 		sprintf(tmpbuf,"Server supports a max mapsize of %d x %d - requesting a %d x %d mapsize",
-			x, y, want_mapx, want_mapy);
+			x, y, use_config[CONFIG_MAPWIDTH], use_config[CONFIG_MAPHEIGHT]);
 		draw_info(tmpbuf,NDI_RED);
 	    }
-	    else if (want_mapx == x && want_mapy == y) {
-		mapx = x;
-		mapy = y;
-		resize_map_window(x,y);
+	    else if (use_config[CONFIG_MAPWIDTH] == x && use_config[CONFIG_MAPHEIGHT] == y) {
+		resize_map_window(use_config[CONFIG_MAPWIDTH], use_config[CONFIG_MAPHEIGHT]);
 	    }
 	    else {
 		/* Our request was not bigger than what server supports, and 
@@ -156,11 +184,10 @@ void SetupCmd(char *buf, int len)
 		 * something is wrong.
 		 */
 		sprintf(tmpbuf,"Unable to set mapsize on server - we wanted %d x %d, server returnd %d x %d",
-			want_mapx, want_mapy, x, y);
+			use_config[CONFIG_MAPWIDTH], use_config[CONFIG_MAPHEIGHT], x, y);
 		draw_info(tmpbuf,NDI_RED);
 	    }
-	} else {
-	    if (!strcmp(cmd,"sexp") || !strcmp(cmd,"darkness") || 
+	} else if (!strcmp(cmd,"sexp") || !strcmp(cmd,"darkness") || 
 		!strcmp(cmd,"newmapcmd") ) {
 	        /* this really isn't an error or bug - in fact, it is expected if
 		 * the user is playing on an older server.
@@ -176,66 +203,45 @@ void SetupCmd(char *buf, int len)
 		    }
 #endif
 		}
-	    } else {
+	} else if (!strcmp(cmd, "facecache")) {
+	    if (!strcmp(param, "FALSE") && want_config[CONFIG_CACHE]) {
+		SendSetFaceMode(csocket, CF_FACE_CACHE);
+	    }
+	    else {
+		use_config[CONFIG_CACHE] = atoi(param);
+	    }
+	} else if (!strcmp(cmd,"faceset")) {
+	    if (!strcmp(param, "FALSE")) {
+		draw_info("Server does not support other image sets, will use default", NDI_RED);
+		face_info.faceset=0;
+	    }
+	} else if (!strcmp(cmd,"map1acmd")) {
+	    if (!strcmp(param,"FALSE")) {
+		draw_info("Server does not support map1acmd, will try map1cmd", NDI_RED);
+		cs_print_string(csocket.fd,"setup map1cmd 1");
+	    }
+	} else if (!strcmp(cmd,"map1cmd")) {
+	    /* Not much we can do about someone playing on an ancient server. */
+	    if (!strcmp(param,"FALSE")) {
+		draw_info("Server does not support map1cmd - This server is too old to support this client!", NDI_RED);
+		close(csocket.fd);
+		csocket.fd = -1;
+	    }
+	} else if (!strcmp(cmd,"itemcmd")) {
+	    /* don't really care - currently, the server will just
+	     * fall back to the item1 transport mode.  If more item modes
+	     * are used in the future, we should probably fall back one at
+	     * a time or the like.
+	     */
+	}
+
+	else {
 		fprintf(stderr,"Got setup for a command we don't understand: %s %s\n",
 		    cmd, param);
-	    }
 	}
     }
 }
 
-/* Move these here - they don't contain any windows dependant
- * code.
- */
-
-/* We only get here if the server believes we are caching images. */
-/* We rely on the fact that the server will only send a face command for
- * a particular number once - at current time, we have no way of knowing
- * if we have already received a face for a particular number.
- */
-
-void FaceCmd(unsigned char *data,  int len)
-{
-    int pnum;
-    char *face;
-
-    /* A quick sanity check, since if client isn't caching, all the data
-     * structures may not be initialized.
-     */
-    if (!display_willcache()) {
-	fprintf(stderr,"Received a 'face' command when we are not caching\n");
-	return;
-    }
-    pnum = GetShort_String(data);
-    face = (char*)data+2;
-    data[len] = '\0';
-
-    finish_face_cmd(pnum, 0, 0, face);
-
-}
-
-void Face1Cmd(unsigned char *data,  int len)
-{
-    int pnum;
-    uint32  checksum;
-    char *face;
-
-    /* A quick sanity check, since if client isn't caching, all the data
-     * structures may not be initialized.
-     */
-    if (!display_willcache()) {
-	fprintf(stderr,"Received a 'face' command when we are not caching\n");
-	return;
-    }
-    pnum = GetShort_String(data);
-    checksum = GetInt_String(data+2);
-    face = (char*)data+6;
-    data[len] = '\0';
-
-    finish_face_cmd(pnum, checksum, 1, face);
-}
-
-
 
 /* Handles when the server says we can't be added.  In reality, we need to
  * close the connection and quit out, because the client is going to close
@@ -297,51 +303,6 @@ void AnimCmd(unsigned char *data, int len)
     LOG(0,"Received animation %d, %d faces\n", anum, animations[anum].num_animations);
 }
 
-#if 0
-void PixMapCmd(unsigned char *data,  int len)
-{  
-    int pnum,plen;
-
-    pnum = GetInt_String(data);
-    plen = GetInt_String(data+4);
-/*    fprintf(stderr,"Recieved pixmap %d, len %d", pnum, plen);*/
-    if (len<8 || (len-8)!=plen) {
-	fprintf(stderr,"PixMapCmd: Lengths don't compare (%d,%d)\n",
-		(len-8),plen);
-	return;
-    }
-    display_newpixmap(pnum,(char*)data+8,plen);
-}
-#endif
-
-void ImageCmd(unsigned char *data,  int len)
-{  
-    int pnum,plen;
-
-    pnum = GetInt_String(data);
-    plen = GetInt_String(data+4);
-/*    fprintf(stderr,"Recieved pixmap %d, len %d", pnum, plen);*/
-    if (len<8 || (len-8)!=plen) {
-	fprintf(stderr,"PixMapCmd: Lengths don't compare (%d,%d)\n",
-		(len-8),plen);
-	return;
-    }
-    display_newpng(pnum,(char*)data+8,plen);
-#if 0
-    /* Currently, the image command should only really be used for 
-     * for png, but it could just as easily get used for xpm with no
-     * significant changes.
-     */
-    if (display_usexpm()) {
-	display_newpixmap(pnum,(char*)data+8,plen);
-    } else if (display_usepng()) {
-	display_newpng(pnum,(char*)data+8,plen);
-    }
-    else {
-	fprintf(stderr,"Image command called with unknown image type.\n");
-    }
-#endif
-}
 
 void DrawInfoCmd(char *data, int len)
 {
@@ -548,48 +509,15 @@ void item_actions (item *op)
 */
 }
 
-/* ItemCmd grabs and display information for items in the inventory */
-void ItemCmd(unsigned char *data, int len)
+
+/* common_item_cmd parses the data send to us from the server.
+ * revision is what item command the data came from - newer
+ * ones have addition fields.
+ */
+static void common_item_command(uint8 *data, int len, int revision)
 {
-    int weight, loc, tag, face, flags,pos=0,nlen;
-    char name[MAX_BUF];
 
-    loc = GetInt_String(data);
-    pos+=4;
-
-    if (pos == len) {
-	/* An empty item command can be used to delete the whole
-	 * inventory of item 'loc'. Esspecially if loc is 0, then
-	 * there is no floor under the player.
-	 */
-/*	delete_item_inventory(loc);*/
-	fprintf(stderr,"ItemCmd: Got location with no other data\n");
-    }
-    else if (loc < 0) { /* delele following items */
-	fprintf(stderr,"ItemCmd: got location with negative value (%d)\n", loc);
-	return;
-    } else {
-	while (pos < len) {
-	    tag=GetInt_String(data+pos); pos+=4;
-	    flags = GetInt_String(data+pos); pos+=4;
-	    weight = GetInt_String(data+pos); pos+=4;
-	    face = GetInt_String(data+pos); pos+=4;
-	    nlen = data[pos++];
-	    memcpy(name, (char*)data+pos, nlen);
-	    pos += nlen;
-	    name[nlen]='\0';
-	    update_item (tag, loc, name, weight, face, flags,0,0,-1);
-	    item_actions (locate_item(tag));
-	}
-	if (pos>len) 
-	    fprintf(stderr,"ItemCmd: Overread buffer: %d > %d\n", pos, len);
-    }
-}
-
-/* ItemCmd grabs and display information for items in the inventory */
-void Item1Cmd(unsigned char *data, int len)
-{
-    int weight, loc, tag, face, flags,pos=0,nlen,anim,nrof;
+    int weight, loc, tag, face, flags,pos=0,nlen,anim,nrof, type;
     uint8 animspeed;
     char name[MAX_BUF];
 
@@ -597,14 +525,10 @@ void Item1Cmd(unsigned char *data, int len)
     pos+=4;
 
     if (pos == len) {
-	/* An empty item command can be used to delete the whole
-	 * inventory of item 'loc'. Esspecially if loc is 0, then
-	 * there is no floor under the player.
-	 */
-/*	delete_item_inventory(loc);*/
 	fprintf(stderr,"ItemCmd: Got location with no other data\n");
+	return;
     }
-    else if (loc < 0) { /* delele following items */
+    else if (loc < 0) { /* delete following items */
 	fprintf(stderr,"ItemCmd: got location with negative value (%d)\n", loc);
 	return;
     } else {
@@ -620,8 +544,12 @@ void Item1Cmd(unsigned char *data, int len)
 	    anim = GetShort_String(data+pos); pos+=2;
 	    animspeed = data[pos++];
 	    nrof = GetInt_String(data+pos); pos+=4;
+	    if (revision >= 2) {
+		type = GetShort_String(data+pos); pos+=2;
+	    } else
+		type = NO_ITEM_TYPE;
 	    update_item (tag, loc, name, weight, face, flags, anim,
-			 animspeed, nrof);
+			 animspeed, nrof, type);
 	    item_actions (locate_item(tag));
 	}
 	if (pos>len) 
@@ -629,6 +557,20 @@ void Item1Cmd(unsigned char *data, int len)
     }
 }
 
+/* parsing the item data is 95% the same - we just need to pass the
+ * revision so that the function knows what values to extact.
+ */
+void Item1Cmd(unsigned char *data, int len)
+{
+    common_item_command(data, len, 1);
+}
+
+void Item2Cmd(unsigned char *data, int len)
+{
+    common_item_command(data, len, 2);
+}
+
+
 /* UpdateItemCmd updates some attributes of an item */
 void UpdateItemCmd(unsigned char *data, int len)
 {
@@ -704,7 +646,7 @@ void UpdateItemCmd(unsigned char *data, int len)
      * calling update_item is a little bit of overkill, since we
      * already determined some of the values in this function.
      */
-    update_item (tag, loc, name, weight, face, flags,anim,animspeed,nrof);
+    update_item (tag, loc, name, weight, face, flags,anim,animspeed,nrof, ip->type);
     item_actions (locate_item(tag));
 }
 
@@ -732,114 +674,300 @@ void DeleteInventory(unsigned char *data, int len)
     remove_item_inventory(locate_item(tag));
 }
 
+/******************************************************************************
+ * Start of map commands
+ *****************************************************************************/
+
+/* expand_face sets (or clears) big_face information.  IT basically starts
+ * at hx,hy (in the_map spaces), and expands to dx,dy (in the negative direction.)
+ * it looks in these spaces for values that matches oldface.  If it finds
+ * such a value, it sets it with newface.
+ * if oldface is zero and newface is nonzero, it then sets the size_x and size_y
+ * values to positive values which point to where the base of the face is -
+ * this is used for drawing.
+ * If oldface is nonzero and newface is zero, then it only clears
+ * those values in which the size_ values match.
+ * layer is the preferred layer that a new face should be put on.
+ */
+static void expand_face(int hx, int hy, int dx, int dy, int oldface, int newface, int layer)
+{
+    int x, y, i, zx, zy, l;
+
+/*
+    fprintf(stderr,"expand_face: hx=%d, hy=%d, dx=%d, dy=%d, old=%d, new=%d, layer=%d\n",
+	    hx, hy, dx, dy, oldface, newface,layer);
+*/
+
+    for (x=0; x<dx; x++) {
+	zx = hx - x;
+	if (zx < 0) zx+= the_map.x;
+	for (y=0; y<dy; y++) {
+	    if (x==0 && y==0) continue;	    /* Special - don't expand into ourselves */
+	    zy = hy - y;
+	    if (zy < 0) zy += the_map.y;
+
+	    /* This loop here tries to find a tails slot to put this info
+	     * into.  we use l as the offset to better try and match the layer
+	     * to store this on.
+	     */
+	    for (i=0; i < MAXLAYERS; i++) {
+		l = (i + layer) % MAXLAYERS;
+
+		/* We try to match oldface - this can be non
+		 * zero when we are removing a big image - in that case, we need
+		 * to make sure the size values are the same.
+		 */
+		if (the_map.cells[zx][zy].tails[l].face == oldface) {
+		    if (oldface == 0 || 
+			(the_map.cells[zx][zy].tails[l].size_x == x && 
+			 the_map.cells[zx][zy].tails[l].size_y == y))
+			    break;
+		}
+		/* If we already have this face information, break out - the code below
+		 * will fill in the same values, but that isn't any big deal.
+		 */
+		if (the_map.cells[zx][zy].tails[l].face == newface && 
+		    the_map.cells[zx][zy].tails[l].size_x == x && 
+		    the_map.cells[zx][zy].tails[l].size_y == y) break;
+
+	    }
+	    /* Not found - either no space for a new face, or the oldface was already cleared out */
+	    if (i>= MAXLAYERS) {
+		fprintf(stderr,"expand_face: Did not find empty slot\n");
+		return;
+	    }
+
+	    the_map.cells[zx][zy].tails[l].face = newface;
+	    /* only set the size_ values if there is an actual image */
+	    if (newface) {
+		the_map.cells[zx][zy].tails[l].size_x = x;
+		the_map.cells[zx][zy].tails[l].size_y = y;
+	    }
+	    /* Else setting to empty face */
+	    else {
+		the_map.cells[zx][zy].tails[l].size_x = 0;
+		the_map.cells[zx][zy].tails[l].size_y = 0;
+	    }
+	    the_map.cells[zx][zy].need_update = 1;
+	} /* for y */
+    } /* for x */
+}
+
+/* This does the task of actually clearing the data in a cell.
+ * This is called if this space was cleared but we still wanted
+ * to display the data for fog of war code.  This is called
+ * when we have real data to over-write the contents of this
+ * cell.
+ */
+void reset_cell_data(int x, int y)
+{
+
+    /* This cell has been cleared previously but now we are 
+     * writing new data to do. So we have to clear it for real now 
+     */
+    int i= 0;
+
+    the_map.cells[x][y].darkness= 0;
+    the_map.cells[x][y].need_update= 1;
+    the_map.cells[x][y].have_darkness= 0;
+    the_map.cells[x][y].cleared= 0;
+    /* only clear the even faces - those explicitly sent to us by the server */
+    for (i=0; i<MAXLAYERS; i++) {
+	/* If we are clearing out a 'big image', we need to clear the faces that
+	 * this points into.
+	 */
+	if (the_map.cells[x][y].heads[i].face != 0 &&
+	    (the_map.cells[x][y].heads[i].size_x>1 || the_map.cells[x][y].heads[i].size_y>1))
+	    expand_face(x, y, the_map.cells[x][y].heads[i].size_x, the_map.cells[x][y].heads[i].size_y,
+			the_map.cells[x][y].heads[i].face, 0, i);
+
+	the_map.cells[x][y].heads[i].face= 0;  /* empty/blank face */
+	the_map.cells[x][y].heads[i].size_x = 0;
+	the_map.cells[x][y].heads[i].size_y = 0;
+    }
+}
+/* Modified the logic of this.  Basically, we always act as if
+ * we are using the fog mode in terms of clearing the data.  We'll
+ * leave it to the display logic to determine if it should draw
+ * the space black or draw it in fog fashion.
+ * This makes the logic much easier when dealing with the big images,
+ * as we sort of need to track that a portion of a multipart image is
+ * in fact on that space, because when the space re-appears in view,
+ * we may not know otherwise know that we need to update that space
+ * again.
+ */
+void display_map_clearcell(int x,int y)
+{
+
+    int i,got_one=0;
+
+    x+= pl_pos.x;
+    y+= pl_pos.y;
+
+    /* If the space is completly blank, don't mark this as a 
+     * fog cell - that chews up extra cpu time.  Likewise,
+     * if the space is completely dark, don't draw it either.
+     * We do check the odd values (heads on other spaces),
+     * because we are only checking to see if we should
+     * in fact draw this space.
+     */
+    for (i=0; i<MAXLAYERS; i++) {
+	if (the_map.cells[x][y].heads[i].face>0) got_one=1;
+	if (the_map.cells[x][y].tails[i].face>0) got_one=1;
+    }
+
+    /* This spell is either already blank or completely dark */
+#if 0
+    if (!got_one || the_map.cells[x][y].darkness>200) {
+#else
+    if (!got_one) {
+#endif
+	reset_cell_data(x,y);
+    }
+    /* else this space has stuff we want to draw */
+    else {
+	the_map.cells[x][y].cleared= 1;
+	the_map.cells[x][y].need_update= 1;
+    }
+    return;
+}
+
+
+/* sets the face at layer to some value.  We just can't
+ * restact arbitrarily, as the server now sends faces only
+ * for layers that change, and not the entire space.
+ * offx and offy are typically zero - this is where the head
+ * of the face is located.  This can be used by the client
+ * to only blit the portions on the actual space.  This
+ * is very important in the case where the head may be off the
+ * viewable area, but a portion is visible.
+ */
+static void set_map_face(int x, int y, int layer, int face)
+{
+    x+= pl_pos.x;
+    y+= pl_pos.y;
+
+    /* this space is getting changed.  If the old face was a 'big face',
+     * we need to clear the spaces that we were set to draw.
+     * This loop basically just looks through the spaces that the face
+     * we are clearing out extends to, and marks them as blank.
+     * Set the new face to zero - the logic below will re-set this face
+     * if this is a case of a face changing (eg, animated)
+     */
+
+    if (the_map.cells[x][y].heads[layer].face != 0 &&
+       (the_map.cells[x][y].heads[layer].size_x>1 || the_map.cells[x][y].heads[layer].size_y>1))
+	    expand_face(x, y, the_map.cells[x][y].heads[layer].size_x, the_map.cells[x][y].heads[layer].size_y,
+		    the_map.cells[x][y].heads[layer].face, 0, layer);
+
+
+    the_map.cells[x][y].heads[layer].face = face;
+    get_map_image_size(face, 
+		    &the_map.cells[x][y].heads[layer].size_x, 
+		    &the_map.cells[x][y].heads[layer].size_y);
+
+    the_map.cells[x][y].need_update = 1;
+    the_map.cells[x][y].have_darkness = 1;
+}
+
+
 void NewmapCmd(unsigned char *data, int len)
 {
     display_map_newmap();
 }
 
-void Map_unpacklayer(unsigned char *cur,unsigned char *end)
+
+/* This is the common processing block for the map1 and
+ * map1a protocol commands.  The map1a mieks minor extensions
+ * and are easy to deal with inline (in fact, this code
+ * doesn't even care what rev is - just certain bits will
+ * only bet set when using the map1a command.
+ * rev is 0 for map1,
+ * 1 for map1a.  It conceivable that there could be future
+ * revisions.
+ */
+#define NUM_LAYERS 2
+static void map1_common(unsigned char *data, int len, int rev)
 {
-  long x,y,face;
-  int xy;
-  int clear = 1;
-  unsigned char *fcur;
-  
-  while (cur < end && *cur != 255) {
-    xy = *cur;
-    cur++;
-    x = xy / mapy;
-    y = xy % mapy;
-    display_map_clearcell(x,y);
-  }
-  cur++;
-  while(cur < end) {
-    fcur = cur;
-    face = *cur & 0x7f;
-    cur++;
-    face = (face << 8) | *cur;
-    cur++;
-
-    /* Now process all the spaces with this face */
-    while(1) {
-      xy = *cur & 0x7f;
-      x = xy / mapy;
-      y = xy % mapy;
-      if (clear) {
-	display_map_clearcell(x,y);
-      }
-      display_map_addbelow(x,y,face);
-      xy = *cur;
-      cur++;
-      if (xy & 128) {
-	break;
-      }
-      if (cur==end) {
-	fprintf(stderr,"Incorrectly packed map.\n");
-	abort();
-      }
-    }
-    /* Got end of layer */
-    if (*fcur & 128) 
-      clear = 0; /* end of first layer (or some layer after the first) */
-  }
-}
-
-
-void MapCmd(unsigned char *data, int len)
-{
-    unsigned char *end;
-
-#if 0
-    fprintf(stderr,"Got MapCmd - %d bytes\n", len);
-#endif
-    map1cmd=0;
-    end = data + len;
-
-    display_map_startupdate();
-    Map_unpacklayer(data,end);
-    display_map_doneupdate(FALSE);
-}
-
-void Map1Cmd(unsigned char *data, int len)
-{
-    int mask, x, y, pos=0,face;
+    int mask, x, y, pos=0,face, layer;
 
     map1cmd=1;
-#if 0
-    fprintf(stderr,"Got Map1Cmd - %d bytes\n", len);
-#endif
     display_map_startupdate();
+/*    fprintf(stderr,"map1 bytes %d\n", len);*/
 
     while (pos <len) {
 	mask = GetShort_String(data+pos); pos+=2;
 	x = (mask >>10) & 0x3f;
 	y = (mask >>4) & 0x3f;
+
+
 	/* If there are no low bits (face/darkness), this space is
 	 * not visible.
 	 */
 	if ((mask & 0xf) == 0) {
 	    display_map_clearcell(x,y);
+	    continue;	/* if empty mask, none of the checks will be true. */
 	}
+	/* If this space was previously stored for fog of war, need to clear
+	 * it now.  Needs to be done before anything else happens that we
+	 * might want to store in it.
+	 */
+
+	if(the_map.cells[x+pl_pos.x][y+pl_pos.y].cleared == 1) {
+	    reset_cell_data(x+pl_pos.x, y+pl_pos.y);
+	}
+
 	if (mask & 0x8) { /* darkness bit */
 	    set_map_darkness(x,y, (uint8)(data[pos]));
 	    pos++;
 	}
-	if (mask & 0x4) { /* floor bit */
-	    face = GetShort_String(data+pos); pos +=2;
-	    set_map_face(x,y,0, face);
+	/* Reduce redundant by putting the get image
+	 * and flags in a common block.  The layers
+	 * are the inverse of the bit order unfortunately.
+	 */
+	for (layer=NUM_LAYERS; layer>=0; layer--) {
+	    if (mask & (1 << layer)) {
+		face = GetShort_String(data+pos); pos +=2;
+		set_map_face(x, y, NUM_LAYERS - layer, face);
+	    }
 	}
-	if (mask & 0x2) { /* middle face */
-	    face = GetShort_String(data+pos); pos +=2;
-	    set_map_face(x,y,1, face);
-	}
-	if (mask & 0x1) { /* top face */
-	    face = GetShort_String(data+pos); pos +=2;
-	    set_map_face(x,y,2, face);
+
+	/* If this is a big space, need to tell spaces that this extends into about it.
+	 * We do this this after filling in all the faces for the space.  This
+	 * is necessary because expand_face collapses duplicates.  But we can get
+	 * into cases were layer 2 adds a space and layer 1 removes it because it used
+	 * to be on layer 1.  This results in it completely disappearing.  So
+	 * we do this here, because layer 2 still has the right value.
+	 */
+	x+= pl_pos.x;
+	y+= pl_pos.y;
+	for (layer=0; layer<MAXLAYERS; layer++) {
+	    if (the_map.cells[x][y].heads[layer].face &&
+		(the_map.cells[x][y].heads[layer].size_x>1 || the_map.cells[x][y].heads[layer].size_y>1))
+		expand_face(x, y, the_map.cells[x][y].heads[layer].size_x,
+		    the_map.cells[x][y].heads[layer].size_y,
+		    0, the_map.cells[x][y].heads[layer].face, layer);
 	}
     }
     display_map_doneupdate(FALSE);
+
 }
 
+/* These wrapper functions actually are not needed since
+ * the logic above doesn't care what the revision actually
+ * is.
+ */
+
+void Map1Cmd(unsigned char *data, int len)
+{
+    map1_common(data, len, 0);
+}
+void Map1aCmd(unsigned char *data, int len)
+{
+    map1_common(data, len, 1);
+}
+
+
 void map_scrollCmd(char *data, int len)
 {
     int dx,dy;
