@@ -3,31 +3,43 @@ extern void DoClient(ClientSocket *csocket);
 extern int init_connection(char *host, int port);
 extern void negotiate_connection(int sound);
 /* commands.c */
+extern void ReplyInfoCmd(char *buf, int len);
 extern void SetupCmd(char *buf, int len);
-extern void FaceCmd(unsigned char *data, int len);
-extern void Face1Cmd(unsigned char *data, int len);
 extern void AddMeFail(char *data, int len);
 extern void AddMeSuccess(char *data, int len);
 extern void GoodbyeCmd(char *data, int len);
 extern void AnimCmd(unsigned char *data, int len);
-extern void ImageCmd(unsigned char *data, int len);
 extern void DrawInfoCmd(char *data, int len);
 extern void StatsCmd(unsigned char *data, int len);
 extern void handle_query(char *data, int len);
 extern void send_reply(char *text);
 extern void PlayerCmd(unsigned char *data, int len);
 extern void item_actions(item *op);
-extern void ItemCmd(unsigned char *data, int len);
 extern void Item1Cmd(unsigned char *data, int len);
+extern void Item2Cmd(unsigned char *data, int len);
 extern void UpdateItemCmd(unsigned char *data, int len);
 extern void DeleteItem(unsigned char *data, int len);
 extern void DeleteInventory(unsigned char *data, int len);
+extern void reset_cell_data(int x, int y);
+extern void display_map_clearcell(int x, int y);
 extern void NewmapCmd(unsigned char *data, int len);
-extern void Map_unpacklayer(unsigned char *cur, unsigned char *end);
-extern void MapCmd(unsigned char *data, int len);
 extern void Map1Cmd(unsigned char *data, int len);
+extern void Map1aCmd(unsigned char *data, int len);
 extern void map_scrollCmd(char *data, int len);
 extern void MagicMapCmd(unsigned char *data, int len);
+/* image.c */
+extern void init_common_cache_data(void);
+extern void requestface(int pnum, char *facename);
+extern void finish_face_cmd(int pnum, uint32 checksum, int has_sum, char *face, int faceset);
+extern void reset_image_cache_data(void);
+extern void FaceCmd(unsigned char *data, int len);
+extern void Face1Cmd(unsigned char *data, int len);
+extern void Face2Cmd(uint8 *data, int len);
+extern void ImageCmd(uint8 *data, int len);
+extern void Image2Cmd(uint8 *data, int len);
+extern void display_newpng(long face, uint8 *buf, long buflen, int setnum);
+extern void get_image_info(char *data, int len);
+extern void get_image_sums(uint8 *data, int len);
 /* init.c */
 extern void VersionCmd(char *data, int len);
 extern void SendVersion(ClientSocket csock);
@@ -45,12 +57,12 @@ extern item *locate_item(sint32 tag);
 extern void remove_item(item *op);
 extern void remove_item_inventory(item *op);
 extern item *create_new_item(item *env, sint32 tag);
-extern void set_item_values(item *op, char *name, sint32 weight, uint16 face, uint16 flags, uint16 anim, uint16 animspeed, sint32 nrof);
+extern void set_item_values(item *op, char *name, sint32 weight, uint16 face, uint16 flags, uint16 anim, uint16 animspeed, sint32 nrof, uint16 type);
 extern void toggle_locked(item *op);
 extern void send_mark_obj(item *op);
 extern item *player_item(void);
 extern item *map_item(void);
-extern void update_item(int tag, int loc, char *name, int weight, int face, int flags, int anim, int animspeed, int nrof);
+extern void update_item(int tag, int loc, char *name, int weight, int face, int flags, int anim, int animspeed, int nrof, int type);
 extern void print_inventory(item *op);
 extern void animate_objects(void);
 /* metaserver.c */
